@@ -1098,6 +1098,14 @@ static gboolean
 srl_report_obj_event_to_clients (gint index,
 				 SREventType type)
 {
+    gint i;
+
+    srl_assert (0 <= index && index < SRL_EVENT_PRIORITY_LAST);
+
+    for (i = index + 1; i < SRL_EVENT_PRIORITY_LAST; i++)
+	if (srl_last_events[i] != NULL)
+	    return TRUE;
+
     srl_assert (0 <= index && index < SRL_EVENT_PRIORITY_LAST);
 
     srl_report_event_from_lasts_to_clients (index, type,
@@ -1283,6 +1291,162 @@ srl_event_is_for_metacity (SRLEvent *event)
     return rv;
 }
 
+static gboolean
+srl_is_label_for (Accessible *label, Accessible *acc)
+{
+    AccessibleRelation **relation;
+    gint i;
+    gboolean rv = FALSE;
+
+    srl_assert (acc && label);
+
+    relation = Accessible_getRelationSet (acc);
+    if (!relation)
+	return FALSE;
+    for (i = 0; !rv && relation[i]; ++i)
+    {
+	if (AccessibleRelation_getRelationType (relation[i]) == SPI_RELATION_LABELED_BY)
+	{
+	    gint j, cnt;
+	    cnt = AccessibleRelation_getNTargets (relation[i]);
+	    for (j = 0; !rv && j < cnt; j++)
+	    {
+		Accessible *label_;
+		label_ = AccessibleRelation_getTarget (relation[i], j);
+		if (label_ == label)
+		    rv = TRUE;
+		if (label_)
+		    Accessible_unref (label_);
+	    }
+	}
+    }
+    for (i = 0; relation[i]; ++i)
+	AccessibleRelation_unref (relation[i]);
+    g_free (relation);
+
+    return rv;
+}
+
+
+
+#define SRL_MAX_LEVELS_UP 5
+static Accessible*
+srl_get_context (Accessible *acc)
+{
+    Accessible *context;
+    AccessibleRole acc_role;
+    gint i;
+
+    sru_assert (acc);
+
+    context = NULL;
+    Accessible_ref (acc);
+    acc_role = Accessible_getRole (acc);
+    if (acc_role == SPI_ROLE_TABLE && acc != srl_last_table)
+    {
+	context = acc;
+	Accessible_ref (context);
+    }    
+    for (i = 0; i < SRL_MAX_LEVELS_UP; i++)
+    {
+	Accessible *parent = Accessible_getParent (acc);
+	if (parent)
+	{
+	    AccessibleRole role;
+	    Accessible_unref (acc);
+	    acc = parent;
+	    role = Accessible_getRole (acc);
+	    if (role == SPI_ROLE_EMBEDDED || role == SPI_ROLE_TOOL_BAR)
+	    {
+		context = acc;
+		Accessible_ref (context);
+	    }
+	    if (srl_acc_has_state (acc, SPI_STATE_VERTICAL) && 
+		    (role == SPI_ROLE_FILLER || role == SPI_ROLE_PANEL))
+	    {
+		gint cnt;
+		cnt = Accessible_getChildCount (acc);
+		if (cnt == 2)
+		{
+		    Accessible *label, *container;
+		    label = Accessible_getChildAtIndex (acc, 0);
+		    container = Accessible_getChildAtIndex (acc, 1);
+		    if (label && container)
+		    {
+			if (Accessible_getRole (label) == SPI_ROLE_LABEL &&
+			    (Accessible_getRole (container) == SPI_ROLE_FILLER ||
+			     Accessible_getRole (container) == SPI_ROLE_PANEL))
+			{
+			    gchar *name = Accessible_getName (label);
+			    if (name && name[0])
+				if (!srl_is_label_for (label, acc))
+				{
+				    context = label;
+				    Accessible_ref (label);
+				}
+			    SPI_freeString (name);
+			}
+		    }
+		    if (label)
+		        Accessible_unref (label);
+		    if (container)
+		        Accessible_unref (container);
+		}
+	    }
+	    if (role == SPI_ROLE_WINDOW && acc_role == SPI_ROLE_STATUS_BAR)
+	    {
+		context = acc;
+		Accessible_ref (context);
+	    }
+	}
+	else
+	    break;
+	if (context)
+	    break;
+    }
+    if (acc)
+	Accessible_unref (acc);
+    return context;
+}
+
+static gboolean
+srl_check_context_changed (SRLEvent *event)
+{
+    Accessible *context;
+
+    srl_assert (event);
+
+    context = srl_get_context (event->acc_ev->source);
+    if (context)
+    {
+	if (context != srl_last_context)
+	{
+	    SRLEvent *ev;
+	    if (srl_last_context)
+		Accessible_unref (srl_last_context);
+	    srl_last_context = context;
+	    ev = srle_new ();
+	    ev->acc = srl_last_context;
+	    Accessible_ref (srl_last_context);
+	    ev->type = SRL_EVENT_CONTEXT_SWITCHED;
+	    srl_notify_clients_obj (ev, SR_EVENT_WINDOW);
+	    srle_free (ev);
+	}
+	else
+	{
+	    Accessible_unref (context);
+	}
+    }
+    else
+    {
+	if (srl_last_context)
+	    Accessible_unref (srl_last_context);
+	srl_last_context = NULL;    
+    }
+    return TRUE;
+}
+
+
 static gboolean
 srle_change_type (SRLEvent *event)
 {
@@ -1323,7 +1487,10 @@ srle_change_type (SRLEvent *event)
     {
 	if (srle_acc_has_role (event, SPI_ROLE_STATUS_BAR) &&
 	    srl_event_is_for_metacity (event))
+	{
 	    event->type = SRL_EVENT_FOCUS1;
+	    srl_check_context_changed (event);
+	}
 	else
 	{
 	    Accessible *parent = Accessible_getParent (event->acc_ev->source);
@@ -1766,153 +1933,6 @@ srl_set_last_focus2 (Accessible *acc)
     return TRUE;
 }
 
-static gboolean
-srl_is_label_for (Accessible *label, Accessible *acc)
-{
-    AccessibleRelation **relation;
-    gint i;
-    gboolean rv = FALSE;
-
-    srl_assert (acc && label);
-
-    relation = Accessible_getRelationSet (acc);
-    if (!relation)
-	return FALSE;
-    for (i = 0; !rv && relation[i]; ++i)
-    {
-	if (AccessibleRelation_getRelationType (relation[i]) == SPI_RELATION_LABELED_BY)
-	{
-	    gint j, cnt;
-	    cnt = AccessibleRelation_getNTargets (relation[i]);
-	    for (j = 0; !rv && j < cnt; j++)
-	    {
-		Accessible *label_;
-		label_ = AccessibleRelation_getTarget (relation[i], j);
-		if (label_ == label)
-		    rv = TRUE;
-		if (label_)
-		    Accessible_unref (label_);
-	    }
-	}
-    }
-    for (i = 0; relation[i]; ++i)
-	AccessibleRelation_unref (relation[i]);
-    g_free (relation);
-
-    return rv;
-}
-
-
-#define SRL_MAX_LEVELS_UP 5
-static Accessible*
-srl_get_context (Accessible *acc)
-{
-    Accessible *context;
-    gint i;
-
-    sru_assert (acc);
-
-    context = NULL;
-    Accessible_ref (acc);
-
-    if (Accessible_getRole (acc) == SPI_ROLE_TABLE && acc != srl_last_table)
-    {
-	context = acc;
-	Accessible_ref (context);
-    }    
-    for (i = 0; i < SRL_MAX_LEVELS_UP; i++)
-    {
-	Accessible *parent = Accessible_getParent (acc);
-	if (parent)
-	{
-	    AccessibleRole role;
-	    Accessible_unref (acc);
-	    acc = parent;
-	    role = Accessible_getRole (acc);
-	    if (role == SPI_ROLE_EMBEDDED || role == SPI_ROLE_TOOL_BAR)
-	    {
-		context = acc;
-		Accessible_ref (context);
-	    }
-	    if (srl_acc_has_state (acc, SPI_STATE_VERTICAL) && 
-		    (role == SPI_ROLE_FILLER || role == SPI_ROLE_PANEL))
-	    {
-		gint cnt;
-		cnt = Accessible_getChildCount (acc);
-		if (cnt == 2)
-		{
-		    Accessible *label, *container;
-		    label = Accessible_getChildAtIndex (acc, 0);
-		    container = Accessible_getChildAtIndex (acc, 1);
-		    if (label && container)
-		    {
-			if (Accessible_getRole (label) == SPI_ROLE_LABEL &&
-			    (Accessible_getRole (container) == SPI_ROLE_FILLER ||
-			     Accessible_getRole (container) == SPI_ROLE_PANEL))
-			{
-			    gchar *name = Accessible_getName (label);
-			    if (name && name[0])
-				if (!srl_is_label_for (label, acc))
-				{
-				    context = label;
-				    Accessible_ref (label);
-				}
-			    SPI_freeString (name);
-			}
-		    }
-		    if (label)
-		        Accessible_unref (label);
-		    if (container)
-		        Accessible_unref (container);
-		}
-	    }
-	}
-	else
-	    break;
-	if (context)
-	    break;
-    }
-    if (acc)
-	Accessible_unref (acc);
-    return context;
-}
-
-static gboolean
-srl_check_context_changed (SRLEvent *event)
-{
-    Accessible *context;
-
-    srl_assert (event);
-
-    context = srl_get_context (event->acc_ev->source);
-    if (context)
-    {
-	if (context != srl_last_context)
-	{
-	    SRLEvent *ev;
-	    if (srl_last_context)
-		Accessible_unref (srl_last_context);
-	    srl_last_context = context;
-	    ev = srle_new ();
-	    ev->acc = srl_last_context;
-	    Accessible_ref (srl_last_context);
-	    ev->type = SRL_EVENT_CONTEXT_SWITCHED;
-	    srl_notify_clients_obj (ev, SR_EVENT_WINDOW);
-	    srle_free (ev);
-	}
-	else
-	{
-	    Accessible_unref (context);
-	}
-    }
-    else
-    {
-	if (srl_last_context)
-	    Accessible_unref (srl_last_context);
-	srl_last_context = NULL;    
-    }
-    return TRUE;
-}
 
 
 static gboolean
